'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _reactDom = require('react-dom');

var _reactDom2 = _interopRequireDefault(_reactDom);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * ReactDOMInject
 * Thunk that binds a ReactElement to properties of DOM elements based on selector and returns functions to render to first matching element or all matching elements.
 * You can pass a property with state object that will be injected into the ReactElement as redux state.
 * @param  {ReactElement} ReactElement  The ReactElement to bind properties found in HTML DOM selected elements to.
 * @return {Thunk}                      Returns an object with render and renderAll functions for binding the react element with elements found in the DOM.
 */
var ReactDOMInject = function ReactDOMInject(ReactElement) {
  var _render = function _render(element) {
    var props = arguments.length <= 1 || arguments[1] === undefined ? {} : arguments[1];

    var dataset = element.dataset || datasetShim(element);
    if (dataset) Object.assign(props, dataset);
    var __html = element.innerHTML.length > 0 ? element.innerHTML : null;
    return _reactDom2.default.render(__html ? _react2.default.createElement(ReactElement, _extends({}, props, { dangerouslySetInnerHTML: { __html: __html } })) : _react2.default.createElement(ReactElement, props), element);
  };
  var _tryRender = function _tryRender(selector, props) {
    var element = document.querySelector(selector);
    return element || false;
    if (element) return _render(element, props);
  };
  /** Renders the ReactElement onto the selected element and allows optional default props to be provided. */
  var render = function render(selector) {
    var props = arguments.length <= 1 || arguments[1] === undefined ? {} : arguments[1];

    var rendered = _tryRender(selector, props);
    var attempts = 0;
    var maxAttempts = 20;
    if (rendered) return Promise.resolve(rendered);else {
      return new Promise(function (resolve, reject) {
        attempts++;
        var interval = setInterval(function () {
          var rendered = _tryRender(selector, props);
          if (rendered) {
            clearInterval(interval);
            console.warn('ReactDOMInject: Rendered in ' + attempts + ' attempts => \'' + selector + '\'');
            Promise.resolve(rendered);
          } else if (attempts == maxAttempts) {
            clearInterval(interval);
            var message = 'ReactDOMInject: Failed to find query selector in ' + attempts + ' attempts => \'' + selector + '\'';
            console.warn(message);
            Promise.reject(new Error(message));
          }
        }, 100);
      });
    }
  };
  /** Renders the ReactElement onto all matching elements and allows optional default props to be provided. */
  var renderAll = function renderAll(selector) {
    var props = arguments.length <= 1 || arguments[1] === undefined ? {} : arguments[1];

    var elements = document.querySelectorAll(selector);
    if (elements && elements.length > 0) return Array.from(elements).map(function (x) {
      return _render(x, props);
    });
    console.warn('ReactDOMInject: No matching elements found for query selector => \'' + selector + '\'');
  };
  return { render: render, renderAll: renderAll };
};
exports.default = ReactDOMInject;


var datasetShim = function datasetShim(element) {
  if (!element.hasAttributes()) return;
  var dataset = {};
  var attrs = Array.from(element.attributes);
  var _iteratorNormalCompletion = true;
  var _didIteratorError = false;
  var _iteratorError = undefined;

  try {
    for (var _iterator = attrs[Symbol.iterator](), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
      var attr = _step.value;
      var name = attr.name;
      var value = attr.value;

      if (!name.startsWith('data-')) continue;
      var key = name.split('data-').filter(function (x, i) {
        return i > 0;
      }).map(function (word, i) {
        return i === 0 ? word : word.map(function (letter, position) {
          return position === 0 ? letter.toUpperCase() : letter.toLowerCase();
        });
      });
      dataset[key] = value;
    }
  } catch (err) {
    _didIteratorError = true;
    _iteratorError = err;
  } finally {
    try {
      if (!_iteratorNormalCompletion && _iterator.return) {
        _iterator.return();
      }
    } finally {
      if (_didIteratorError) {
        throw _iteratorError;
      }
    }
  }

  return dataset;
};